/*
 * PlaybackController.h
 *
 *  Created on: 09.07.2010
 *      Author: darkstar
 */

#ifndef PLAYBACKCONTROLLER_H_
#define PLAYBACKCONTROLLER_H_

#include <qobject.h>

#include "media.h"
#include "playlist.h"
#include "playlistaspectlistviewrenderer.h"

class PlaybackController : public QObject
{
	Q_OBJECT
public:
	enum PlayOrder {
		ORDER_NORMAL = 0,
		ORDER_REPEAT_ALL = 1,
		ORDER_REPEAT_ONE = 2,
		ORDER_RANDOM = 3,
		ORDER_REPEAT_RANDOM = 4,
		ORDER_MAX = 5
	};

	PlaybackController();
	virtual ~PlaybackController();

	void attachTo(PlayListAspectListViewRenderer *source);
	void detach();
	PlayListAspectListViewRenderer *aspectSource() { return aspectSource_; }

	void setPlayOrder(PlayOrder);

	// index based methods:
	int activeItemIndex() { return activeItemIndex_; }
	void setActiveItemIndex(int itemIndex);

	int nextActiveItemIndex(bool modifyRandomPlayOrderTableIndex = false);
	int previousActiveItemIndex(bool modifyRandomPlayOrderTableIndex = false);

	// item based methods:
	const PlayListItem &activeItem();
	void setActiveItem(const PlayListItem &item);

	const PlayListItem &nextItem(bool modifyRandomPlayOrderTableIndex = false);
	const PlayListItem &previousItem(bool modifyRandomPlayOrderTableIndex = false);

public slots:
	void stopCurrentPlaybackAndPlayCurrentItem();

	void startOrPauseCurrentPlaybackOrStartNewPlayback();
	void stopCurrentPlayback();
	void playNextItem();
	void playPreviousItem();

signals:
	void activeItemChanged();

private slots:
	void updateView();

private:
	PlayListAspectListViewRenderer *aspectSource_;

	PlayOrder playOrder_;
	int *randomPlayOrderTable;

	int count_;
	int randomPlayOrderTableIndex_;
	int activeItemIndex_;
	PlayListItem activeItem_;

	void updateRandomPlayOrderTable();
	void generateRandomPlayOrderTable();
};

#endif /* PLAYBACKCONTROLLER_H_ */
